@@ -7,9 +7,9 @@ import inspect
 import queue
 import weakref
 from collections import defaultdict
-from collections.abc import Awaitable, Callable
 from threading import Lock, Thread
 from typing import (
+    TYPE_CHECKING,
     Any,
     Concatenate,
     Generic,
@@ -21,12 +21,11 @@ from redux.autorun import Autorun
 from redux.basic_types import (
     Action,
     ActionMiddleware,
-    AutorunArgs,
+    Args,
     AutorunDecorator,
     AutorunOptions,
     AutorunOptionsWithDefault,
     AutorunOptionsWithoutDefault,
-    AutorunOriginalReturnType,
     AutorunReturnType,
     AwaitableOrNot,
     BaseAction,
@@ -35,31 +34,35 @@ from redux.basic_types import (
     CreateStoreOptions,
     DispatchParameters,
     Event,
-    Event2,
     EventHandler,
     EventMiddleware,
     FinishAction,
     FinishEvent,
     InitAction,
     ReducerType,
+    ReturnType,
     SelectorOutput,
     SnapshotAtom,
     State,
+    StrictEvent,
+    SubscribeEventCleanup,
     UnknownAutorunDecorator,
     UnknownViewDecorator,
-    ViewArgs,
     ViewDecorator,
     ViewOptions,
     ViewOptionsWithDefault,
     ViewOptionsWithoutDefault,
-    ViewOriginalReturnType,
     ViewReturnType,
+    WithStateDecorator,
     is_complete_reducer_result,
     is_state_reducer_result,
 )
 from redux.serialization_mixin import SerializationMixin
 from redux.side_effect_runner import SideEffectRunnerThread
 
+if TYPE_CHECKING:
+    from collections.abc import Awaitable, Callable
+
 
 class Store(Generic[State, Action, Event], SerializationMixin):
     """Redux store for managing state and side effects."""
@@ -109,11 +112,11 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         if self.store_options.auto_init:
             if self.store_options.scheduler:
                 self.store_options.scheduler(
-                    lambda: self.dispatch(cast(Action, InitAction())),
+                    lambda: self.dispatch(cast('Action', InitAction())),
                     interval=False,
                 )
             else:
-                self.dispatch(cast(Action, InitAction()))
+                self.dispatch(cast('Action', InitAction()))
 
         if self.store_options.scheduler:
             self.store_options.scheduler(self.run, interval=True)
@@ -145,7 +148,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                     self._call_listeners(self._state)
 
                 if isinstance(action, FinishAction):
-                    self._dispatch([cast(Event, FinishEvent())])
+                    self._dispatch([cast('Event', FinishEvent())])
 
     def _run_event_handlers(self: Store[State, Action, Event]) -> None:
         while len(self._events) > 0:
@@ -213,7 +216,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
     ) -> None:
         for item in items:
             if isinstance(item, BaseAction):
-                action = cast(Action, item)
+                action = cast('Action', item)
                 for action_middleware in self._action_middlewares:
                     action_ = action_middleware(action)
                     if action_ is None:
@@ -222,7 +225,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                 else:
                     self._actions.append(action)
             if isinstance(item, BaseEvent):
-                event = cast(Event, item)
+                event = cast('Event', item)
                 for event_middleware in self._event_middlewares:
                     event_ = event_middleware(event)
                     if event_ is None:
@@ -234,7 +237,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         if self.store_options.scheduler is None and not self._is_running.locked():
             self.run()
 
-    def subscribe(
+    def _subscribe(
         self: Store[State, Action, Event],
         listener: Callable[[State], Any],
         *,
@@ -253,11 +256,11 @@ class Store(Generic[State, Action, Event], SerializationMixin):
 
     def subscribe_event(
         self: Store[State, Action, Event],
-        event_type: type[Event2],
-        handler: EventHandler[Event2],
+        event_type: type[StrictEvent],
+        handler: EventHandler[StrictEvent],
         *,
         keep_ref: bool = True,
-    ) -> Callable[[], None]:
+    ) -> SubscribeEventCleanup:
         """Subscribe to events."""
         if keep_ref:
             handler_ref = handler
@@ -266,12 +269,12 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         else:
             handler_ref = weakref.ref(handler)
 
-        self._event_handlers[cast(Any, event_type)].add(handler_ref)
+        self._event_handlers[cast('Any', event_type)].add(handler_ref)
 
         def unsubscribe() -> None:
-            self._event_handlers[cast(Any, event_type)].discard(handler_ref)
+            self._event_handlers[cast('Any', event_type)].discard(handler_ref)
 
-        return unsubscribe
+        return SubscribeEventCleanup(unsubscribe=unsubscribe, handler=handler)
 
     def _wait_for_store_to_finish(self: Store[State, Action, Event]) -> None:
         """Wait for the store to finish."""
@@ -307,49 +310,49 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         selector: Callable[[State], SelectorOutput],
         comparator: Callable[[State], ComparatorOutput] | None = None,
         *,
-        options: AutorunOptionsWithDefault[AutorunOriginalReturnType],
-    ) -> AutorunDecorator[SelectorOutput, AutorunOriginalReturnType]: ...
+        options: AutorunOptionsWithDefault[ReturnType],
+    ) -> AutorunDecorator[SelectorOutput, ReturnType]: ...
     def autorun(
         self: Store[State, Action, Event],
         selector: Callable[[State], SelectorOutput],
         comparator: Callable[[State], ComparatorOutput] | None = None,
         *,
-        options: AutorunOptions[AutorunOriginalReturnType] | None = None,
+        options: AutorunOptions[ReturnType] | None = None,
     ) -> (
-        AutorunDecorator[SelectorOutput, AutorunOriginalReturnType]
+        AutorunDecorator[SelectorOutput, ReturnType]
         | UnknownAutorunDecorator[SelectorOutput]
     ):
         """Create a new autorun, reflecting on state changes."""
 
         @overload
-        def decorator(
+        def autorun_decorator(
             func: Callable[
-                Concatenate[SelectorOutput, AutorunArgs],
-                AutorunOriginalReturnType,
+                Concatenate[SelectorOutput, Args],
+                ReturnType,
             ],
-        ) -> AutorunReturnType[AutorunOriginalReturnType, AutorunArgs]: ...
+        ) -> AutorunReturnType[ReturnType, Args]: ...
         @overload
-        def decorator(
+        def autorun_decorator(
             func: Callable[
-                Concatenate[SelectorOutput, AutorunArgs],
-                Awaitable[AutorunOriginalReturnType],
+                Concatenate[SelectorOutput, Args],
+                Awaitable[ReturnType],
             ],
-        ) -> AutorunReturnType[Awaitable[AutorunOriginalReturnType], AutorunArgs]: ...
-        def decorator(
+        ) -> AutorunReturnType[Awaitable[ReturnType], Args]: ...
+        def autorun_decorator(
             func: Callable[
-                Concatenate[SelectorOutput, AutorunArgs],
-                AwaitableOrNot[AutorunOriginalReturnType],
+                Concatenate[SelectorOutput, Args],
+                AwaitableOrNot[ReturnType],
             ],
-        ) -> AutorunReturnType[AwaitableOrNot[AutorunOriginalReturnType], AutorunArgs]:
+        ) -> AutorunReturnType[AwaitableOrNot[ReturnType], Args]:
             return Autorun(
                 store=self,
                 selector=selector,
                 comparator=comparator,
-                func=cast(Callable, func),
+                func=cast('Callable', func),
                 options=options or AutorunOptions(),
             )
 
-        return decorator
+        return autorun_decorator
 
     @overload
     def view(
@@ -363,58 +366,90 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         self: Store[State, Action, Event],
         selector: Callable[[State], SelectorOutput],
         *,
-        options: ViewOptionsWithDefault[ViewOriginalReturnType],
-    ) -> ViewDecorator[SelectorOutput, ViewOriginalReturnType]: ...
+        options: ViewOptionsWithDefault[ReturnType],
+    ) -> ViewDecorator[SelectorOutput, ReturnType]: ...
     def view(
         self: Store[State, Action, Event],
         selector: Callable[[State], SelectorOutput],
         *,
-        options: ViewOptions[ViewOriginalReturnType] | None = None,
+        options: ViewOptions[ReturnType] | None = None,
     ) -> (
-        ViewDecorator[SelectorOutput, ViewOriginalReturnType]
-        | UnknownViewDecorator[SelectorOutput]
+        ViewDecorator[SelectorOutput, ReturnType] | UnknownViewDecorator[SelectorOutput]
     ):
         """Create a new view, throttling calls for unchanged selector results."""
 
         @overload
-        def decorator(
+        def view_decorator(
             func: Callable[
-                Concatenate[SelectorOutput, ViewArgs],
-                ViewOriginalReturnType,
+                Concatenate[SelectorOutput, Args],
+                ReturnType,
             ],
-        ) -> ViewReturnType[ViewOriginalReturnType, ViewArgs]: ...
+        ) -> ViewReturnType[ReturnType, Args]: ...
         @overload
-        def decorator(
+        def view_decorator(
             func: Callable[
-                Concatenate[SelectorOutput, ViewArgs],
-                Awaitable[ViewOriginalReturnType],
+                Concatenate[SelectorOutput, Args],
+                Awaitable[ReturnType],
             ],
-        ) -> ViewReturnType[Awaitable[ViewOriginalReturnType], ViewArgs]: ...
+        ) -> ViewReturnType[Awaitable[ReturnType], Args]: ...
 
-        def decorator(
+        def view_decorator(
             func: Callable[
-                Concatenate[SelectorOutput, ViewArgs],
-                AwaitableOrNot[ViewOriginalReturnType],
+                Concatenate[SelectorOutput, Args],
+                AwaitableOrNot[ReturnType],
             ],
-        ) -> ViewReturnType[AwaitableOrNot[ViewOriginalReturnType], ViewArgs]:
+        ) -> ViewReturnType[AwaitableOrNot[ReturnType], Args]:
             _options = options or ViewOptions()
             return Autorun(
                 store=self,
                 selector=selector,
                 comparator=None,
-                func=cast(Callable, func),
+                func=cast('Callable', func),
                 options=AutorunOptions(
                     default_value=_options.default_value,
                     auto_await=True,
                     initial_call=False,
                     reactive=False,
+                    memoization=_options.memoization,
                     keep_ref=_options.keep_ref,
                     subscribers_initial_run=_options.subscribers_initial_run,
                     subscribers_keep_ref=_options.subscribers_keep_ref,
                 ),
             )
 
-        return decorator
+        return view_decorator
+
+    def with_state(
+        self: Store[State, Action, Event],
+        selector: Callable[[State], SelectorOutput],
+    ) -> WithStateDecorator[SelectorOutput]:
+        """Wrap a function, passing the result of the selector as its first argument.
+
+        Note that it has zero reactivity, it just runs the function with the result
+        of the selector whenever it is called explicitly. To make it reactive, use
+        `autorun` instead.
+
+        This is mostly for improving code oragnization and readability. Directly Using
+        `store._state` is also possible.
+        """
+
+        def with_state_decorator(
+            func: Callable[
+                Concatenate[SelectorOutput, Args],
+                ReturnType,
+            ],
+        ) -> Callable[Args, ReturnType]:
+            def wrapper(*args: Args.args, **kwargs: Args.kwargs) -> ReturnType:
+                if self._state is None:
+                    msg = 'Store has not been initialized yet.'
+                    raise RuntimeError(msg)
+                return func(selector(self._state), *args, **kwargs)
+
+            wrapper.__name__ = f'with_state:{func.__name__}'
+
+            return wrapper
+
+        return with_state_decorator
 
     @property
     def snapshot(self: Store[State, Action, Event]) -> SnapshotAtom:
